//+------------------------------------------------------------------+
//|                                             iPeriodConverter.mq4 |
//|                                        Copyright 2015, komposter |
//|                                         http://www.komposter.me/ |
//+------------------------------------------------------------------+
#property copyright 		"Copyright 2015, komposter"
#property link      		"http://www.komposter.me/"
#property version	 		"1.0"
#property description	""
#property strict

#property indicator_chart_window
#property indicator_buffers 0

#include <WinUser32.mqh>

input int PeriodMultiplier = 2;      // Period multiplier

int      HistoryHandle = -1;
int      NewPeriod = 0;

#define  CHART_CMD_UPDATE_DATA            33324

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
{
   if ( !IsDllsAllowed() ) Alert( "Enable DLL calls to update off-line charts!" );

   return(custom_init());
}

int custom_init()
{
	NewPeriod = Period() * PeriodMultiplier;
	if ( OpenHistoryFile() < 0 ) return(INIT_FAILED);

	WriteHistoryHeader();
	UpdateHistoryFile( Bars-1, true );

	UpdateChartWindow();

   return(INIT_SUCCEEDED);
}

void OnDeinit(const int reason)
{
	custom_deinit();
}

void custom_deinit()
{
   if ( HistoryHandle >= 0 )
   {
      FileClose( HistoryHandle );
      HistoryHandle = -1;
   }
}

//+------------------------------------------------------------------+
//| 
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
{
	int n = CheckNewData();
	if (n < 0) return(rates_total);

	// update history file with new data
	UpdateHistoryFile(n);

	// refresh chart window
	UpdateChartWindow();

	return(rates_total);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OpenHistoryFile()
{
	HistoryHandle = FileOpenHistory( Symbol() + (string)NewPeriod + ".hst", FILE_BIN | FILE_WRITE | FILE_SHARE_WRITE | FILE_SHARE_READ | FILE_ANSI );
	if ( HistoryHandle < 0 )
	{
		Alert( "FileOpenHistory( \"", Symbol() + (string)NewPeriod + ".hst", "\" )", " - Error #", GetLastError() );
		return(INIT_FAILED);
	}

   return (0);
}

int WriteHistoryHeader()
{
   if (HistoryHandle < 0) return (-1);

	//---- Prepare history header
	int      file_version	= 401;
//	string   c_copyright		= "(C)opyright 2003, MetaQuotes Software Corp.";
	string   c_copyright		= "Copyright  2008-2015, komposter";
	string   c_symbol			= _Symbol;
	int      i_period			= NewPeriod;
	int      i_digits			= _Digits;
	int      i_unused[13];	ArrayInitialize(i_unused,0);

	//--- Write history file header
	FileWriteInteger	( HistoryHandle, file_version,LONG_VALUE);
	FileWriteString	( HistoryHandle, c_copyright,64);
	FileWriteString	( HistoryHandle, c_symbol,12);
	FileWriteInteger	( HistoryHandle, i_period,LONG_VALUE);
	FileWriteInteger	( HistoryHandle, i_digits,LONG_VALUE);
	FileWriteInteger	( HistoryHandle, 0,LONG_VALUE);
	FileWriteInteger	( HistoryHandle, 0,LONG_VALUE);
	FileWriteArray		( HistoryHandle, i_unused,0,13);

	FileFlush			( HistoryHandle);

   return (0);
}


static double d_open, d_low, d_high, d_close;
static long d_volume;
static datetime i_time;

void WriteHistoryData()
{
   if (HistoryHandle >= 0)
   {
		MqlRates rate;
		
		rate.time = i_time;
		rate.open = d_open;
		rate.low  = d_low;
		rate.high = d_high;
		rate.close = d_close;
		rate.tick_volume = d_volume;
	   rate.spread = 0;
	   rate.real_volume = 0;
	
		FileWriteStruct( HistoryHandle, rate );
   }
}

int UpdateHistoryFile(int start_pos, bool init = false)
{
	if (HistoryHandle < 0) return (-1);
	
	static ulong last_fpos;
	int i, ps;
	
	// normalize open time
	ps = NewPeriod * 60;   
	i_time = Time[start_pos]/ps;
	i_time *=  ps;
	if (init)
	{
		//first time, init data
		d_open = Open[start_pos];
		d_low = Low[start_pos];
		d_high = High[start_pos];
		d_close = Close[start_pos];
		d_volume = Volume[start_pos];                           
		i = start_pos - 1;
		last_fpos = FileTell(HistoryHandle);
	}
	else
	{
		i = start_pos;
		FileSeek(HistoryHandle,last_fpos,SEEK_SET);
	}
	if (i < 0) return (-1);
	
	int cnt = 0;
	datetime LastBarTime;
	//processing bars
	while (i >= 0) {
	   LastBarTime = Time[i];
	
	   //a new bar
	   if (LastBarTime >=  i_time+ps) {
	      //write the bar data
	      WriteHistoryData();
	      cnt++;
	      i_time = LastBarTime/ps;
	      i_time *= ps;
	      d_open = Open[i];
	      d_low = Low[i];
	      d_high = High[i];
	      d_close = Close[i];
	      d_volume = Volume[i];
	   } else {
	      //no new bar
	      d_volume +=  Volume[i];
	      if (Low[i]<d_low) d_low = Low[i];
	      if (High[i]>d_high) d_high = High[i];
	      d_close = Close[i];      
	   }
	   i--;
	}
	
	//record last_fpos before writing last bar.
	last_fpos = FileTell(HistoryHandle);
	
	WriteHistoryData();
	cnt++;
	d_volume -=  Volume[0];
	
	//flush the data writen
	FileFlush(HistoryHandle);
	
	if ( init )
	{
		Print( "| Open \"", _Symbol, ", M", NewPeriod, "\" chart to view results (", cnt, " bars)!" );
	}
	return (cnt);
}

int UpdateChartWindow()
{
   static int hwnd = 0;

   if (HistoryHandle < 0) {
      //no HST file opened, no need updating.
      return (-1);
   }
   if(hwnd == 0)
   {
      //trying to detect the chart window for updating
      hwnd = WindowHandle(Symbol(), NewPeriod);
		if ( hwnd != 0 ) Print( "| \"", _Symbol, ", M", NewPeriod, "\" detected!" );
   }
   if(hwnd!= 0)
   {
      if (IsDllsAllowed() == false)
      {
         //DLL calls must be allowed
         return (-1);
      }
      if (PostMessageA(hwnd,WM_COMMAND,CHART_CMD_UPDATE_DATA,0) == 0)
      {
         //PostMessage failed, chart window closed
         hwnd = 0;
      } else {
         //PostMessage succeed
         return (0);
      }
   }
   //window not found or PostMessage failed
   return (-1);
}

static datetime LastStartTime = 0;
static datetime LastEndTime = 0;
static int LastBarCount = 0;

void reinit()
{
   custom_deinit();
   custom_init();
   LastStartTime = Time[Bars-1];
   LastEndTime = Time[0];
   LastBarCount = Bars;
}

bool IsDataChanged()
{
   static int LastBars = 0;
   static datetime LastTime = 0;
   static long LastVolume = 0;
   bool ret;
   
   ret = false;
   if (LastVolume != Volume[0]) {
      LastVolume = Volume[0];
      ret = true;
   }
   if (LastTime != Time[0]) {
      LastTime = Time[0];
      ret = true;
   }
   if (LastBars != Bars) {
      LastBars = Bars;
      ret = true;
   }
   return (ret);
}

int CheckNewData()
{
	static string LastServer = "";
	
	if (Bars < 2)
	{
	   //the data is not loaded yet.
	   // DebugMsg("Data not loaded, only " +  Bars + " Bars");
	   return (-1);
	}
	
	string serv = AccountServer();
	if (serv == "")
	{
	   //no server yet
	   // DebugMsg("No server connected");
	   return (-1);
	}
	
	//server changed? check this and reinit to prevent wrong data while changing server.
	if (LastServer != serv) {
	   // DebugMsg("Server changed from " + LastServer + " to " + serv);
	   LastServer = serv;
	   reinit();
	   return (-1);
	}
	
	if ( !IsDataChanged() )
	{
	   //return if no data changed to save resource
	   // DebugMsg("No data changed");
	   return (-1);
	}
	
	if (Time[Bars-1] != LastStartTime)
	{
	   // DebugMsg("Start time changed, new history loaded or server changed");
	   reinit();
	   return (-1);
	}
	
	int i, cnt;
	
	//try to find LastEndTime bar, which should be Time[0] or Time[1] usually,
	//so the operation is fast
	for (i = 0; i < Bars; i++)
	{
	   if (Time[i] <= LastEndTime)
	   {
	      break;
	   }
	}
	
	if (i >= Bars || Time[i] != LastEndTime)
	{
	   // DebugMsg("End time " + TimeToStr(LastEndTime) + " not found");
	   reinit();
	   return (-1);
	}
	
	cnt = Bars - i;
	if (cnt != LastBarCount)
	{
	   // DebugMsg("Data loaded, cnt is " + cnt + " LastBarCount is " + LastBarCount);
	   reinit();
	   return (-1);
	}
	
	//no new data loaded, return with LastEndTime position.
	LastBarCount = Bars;
	LastEndTime = Time[0];
	
	return (i);
}


